﻿using System;
using System.Web.Mvc;
using MVC_App.Models;
using MVC_App.Validation;

namespace MVC_App.Controllers
{
	public partial class HomeController : Controller
	{
		private KreditkarteZahlungModel ValidateKreditkarteZahlung(BestellungForm bestellung)
		{
			var v = ModelStateValidator.Build(ModelState, bestellung).MoveToNode(p => p.Kreditkarte);

			var vName = v.Validate(p => p.Name).NotNullOrWhitespace();
			var vNummer = v.Validate(p => p.Nummer).NotNullOrWhitespace();

			var vMonat = v.Validate(p => p.GültigBisMonat).NotNullOrWhitespace().ToInt32().InRange(1, 12);
			var vJahr = v.Validate(p => p.GültigBisJahr).NotNullOrWhitespace().ToInt32().InRange(2012, 2020);

			var vGültigkeit = ValidationUtil.JoinGuardEx(vMonat, vJahr,
															(monat, jahr) => new DateTime(jahr, monat, 1),
															"Ungültiger Zeitraum");

			KreditkarteZahlungModel validateKreditkarteZahlung = ValidationUtil.JoinGuardEx(
				vName, vNummer, vGültigkeit,
				(name, nummer, gültigkeit) => KreditkarteServiceGateway.CheckValidity(name, nummer, gültigkeit),
				"Die Kreditkarteninformationen sind ungültig");

			return validateKreditkarteZahlung;
		}
	}

	public static class ValidationUtil
	{
		public static ValidatingNode<TResult> JoinGuardEx<T1, T2, TResult>(
			ValidatingNode<T1> v1,
			ValidatingNode<T2> v2,
			Func<T1, T2, TResult> transformer,
			string errorMessage)
		{
			var merge = v1._errorWrapper.Merge(v2._errorWrapper);
			if (v1.DoEvaluate && v2.DoEvaluate)
			{
				try
				{
					var result = transformer(v1.Value, v2.Value);
					return new ValidatingNode<TResult>(result, merge);
				}
				catch (Exception)
				{
					return new ValidatingNode<TResult>(merge.SetError(errorMessage));
				}
			}
			return new ValidatingNode<TResult>(merge);
		}

		public static ValidatingNode<TResult> JoinGuardEx<T1, T2, T3, TResult>(
			ValidatingNode<T1> v1,
			ValidatingNode<T2> v2,
			ValidatingNode<T3> v3,
			Func<T1, T2, T3, TResult> transformer,
			string errorMessage)
		{
			var merge = v1._errorWrapper.Merge(v2._errorWrapper).Merge(v3._errorWrapper);
			if (v1.DoEvaluate && v2.DoEvaluate)
			{
				try
				{
					var result = transformer(v1.Value, v2.Value, v3.Value);
					return new ValidatingNode<TResult>(result, merge);
				}
				catch (Exception)
				{
					return new ValidatingNode<TResult>(merge.SetError(errorMessage));
				}
			}
			return new ValidatingNode<TResult>(merge);
		}
	}

}